window.addEventListener('load', function() {

    let rows = 0;
    let cols = 0;
    let elements = [];
    let feld = document.getElementById("feld");
    let colors = ["black", "red", "yellow", "green", "cyan", "blue", "magenta", "orange"];
    let classes = []

    function init(r,c) {
        rows = r;
        cols = c;
        feld.style.gridTemplateColumns = `repeat(${cols}, 1fr)`
        for (r=0; r<rows; r++) {
            elements[r] = [];
            for (c=0; c<cols; c++) {
                let div = document.createElement('div');
                div.setAttribute('spalte', c);
                div.setAttribute('zeile', r);
                div.setAttribute('wert', 0);
                div.classList.add("feld"+colors[0])
                feld.appendChild(div);
                elements[r][c] = div;
                set(r,c,0)
            }
        }
        document.styleSheets[0].insertRule(`#feld div{width:min(${90/cols}vw, ${90/rows}vh);}`)
        colors.forEach((color,i) => {
            let c = `feld${color}`
            classes.push(c);
            document.styleSheets[0].insertRule(`.${c} { background-color:${color};}`)
        })
    }

    function set(row, col, val) {
        if (!on(row, col)) return; // Ausserhalb vom Feld
        val = Number(val)
        let el = elements[row][col]
        let old = Number(el.getAttribute('wert'))
        if (old==val) return;  // Kein neuer Wert
        if (old>=0 && old<classes.length) {
            el.classList.remove(classes[old])
        }
        if (val>=0 && val<classes.length) {
            el.setAttribute('wert', val)
            //console.log(el)
            el.classList.add(classes[val])
        }
    }

    function on(row, col) {
        // if (row===undefined || col===undefined) throw "WAT?"
        return !(row<0 || col<0 || row>=rows || col>=cols)
    }

    function get(row, col) {
        if (!on(row, col)) return undefined
        //console.log(`row=${row}, col=${col}`)
        return Number(elements[row][col].getAttribute('wert'))
    }

    let tetris = {
        /* 4x3 grid:
        8  9 10 11
        4  5  6  7
        0  1  2  3
        */
        stones : [[0,1,2,3], [4,0,1,2], [0,1,2,6],[0,1,5,6], [4,5,1,2],[0,1,2,5], [0,1,4,5]],
        stone : 0,
        current : [],  // relative Positionen der Steinquadrate
        x : 0,   // Absolute Position vom Stein
        y : 0,   
        speed : 400,  // ms for next step

        stoneOk : function() {
            for (p of this.current) {
                let [r,c] = [p[1]+this.y, p[0]+this.x];
                if (!on(r,c)) return false;
                if (get(r,c)!=0) return false
            }
            return true;
        },

        nextStone : function() {
            this.stone = Math.floor(Math.random()*this.stones.length)
            this.current = this.stones[this.stone].map((i) => [i%4, Math.floor(i/4)]);
            this.x = 4
            this.y = 0
            return this.stoneOk()
        },

        paint : function(draw=1) {
            for (p of this.current) {
                let [r,c] = [p[1]+this.y,  p[0]+this.x];
                set(r,c,(this.stone+1)*draw)
            }
        },

        stoneMove : function(dx, dy) {
            this.paint(0)  // remove stone
            this.y+=dy      // translate
            this.x+=dx 
            let ok = this.stoneOk()
            if (!ok) {
                this.x-=dx      // back up
                this.y-=dy      // back up
            }
            this.paint()   // place stone again
            return ok
        },

        rotate : function(sense=1) {
            this.paint(0)
            let orig = this.current
            this.current = this.current.map((p)=>[-p[1]*sense, p[0]*sense]);
            let mins = [0,1].map((i) => Math.min(...(this.current.map((p)=>p[i]))) )
            this.current = this.current.map((p)=>[p[0]-mins[0], p[1]-mins[1]])
            let ok = this.stoneOk()
            if (!ok) {
                this.current = orig
            }
            this.paint()
            return ok
        },

        toHandler : undefined,        
        step : function() {
            if (this.toHandler !== undefined) {
                clearTimeout(this.toHandler)
                this.toHandler = undefined
            }
            if (!this.stoneMove(0,1)) {
                if (!this.nextStone()) {
                    console.log("Bust!")
                    return
                    this.reset()
                }
            }
            this.toHandler = setTimeout(() => {
                tetris.step()
            }, this.speed);
        },

        reset : function() {
            field_init()
            this.nextStone()
        },


        init : function() {
            this.reset();
            window.addEventListener('keydown', (ev) => {
                switch (ev.key) {
                    case ' ':
                        ok = this.step()
                        break
                    case 'ArrowLeft':
                        this.stoneMove(-1,0)
                        break
                    case 'ArrowRight':
                        this.stoneMove(1,0)
                        break
                    case 'ArrowUp':
                        this.rotate(1)
                        break
                    case 'ArrowDown' :
                        this.rotate(-1)
                        break
                }
            })
        },

    }

    function field_init() {
        for (let r=0; r<rows; r++) {
            for (let c=0; c<cols; c++) {
                set(r,c,0)
            }
        }
    }

    init(20,10)
    tetris.init()
    tetris.step()
    
})